import reqman, asyncio,pytest
from pprint import pprint

MOCK={
    "/a":(200,"ok"),
}

def test_simple(exe):  
    with open("f.yml","w+") as fid:
        fid.write("""
- GET: /a
  tests:
    - status: 200
- GET: /<<i|method>>
  headers:
    x-hello: world
  params:
    i: 0
    method: return 12/x
  tests:
    - status: 200
- GET: /<<i|method>>
  headers:
    x-hello: world
  params:
    i: 0
  tests:
    - status: 200
- GET: /a
  headers:
    x-hello: <<i|method>>
  params:
    i: 0
  tests:
    - status: 200
- POST: /a
  body: <<i|method>>
  headers:
    x-hello: world
  params:
    i: 0
    method: return 12/x
  tests:
    - status: 200

""")
        
    x=exe(".",fakeServer=MOCK)
    # x.view()
    assert x.rc == 4

def test_py_in_save(exe):  
    with open("f.yml","w+") as fid:
        fid.write("""
- GET: /a
  tests:
    - status: 200
- GET: /a
  save: 
    r: <<i|method>>
  params:
    i: 0
    method: return 12/x
  tests:
    - status: 200
""")
        
    x=exe(".",fakeServer=MOCK)
    # x.view()
    assert x.rc == 1

def test_py_in_foreach(exe):  
    with open("f.yml","w+") as fid:
        fid.write("""
- GET: /a
  tests:
    - status: 200
- GET: /a
  foreach: <<i|method>>
  params:
    i: 0
    method: return 12/x
  tests:
    - status: 200
""")
        
    x=exe(".",fakeServer=MOCK)
    assert x.rc == -1 # real error



import pytest, reqman, json
import datetime,pickle


def test_prettify():
    print( reqman.prettify("<root><a>yolo</a></root>") )
    print( reqman.prettify("<root><a>yolo</a>") )
    print( reqman.prettify(json.dumps( {"a":dict(a=1,b=2),"b":list("abc")}) ) )
    print( reqman.prettify( bytes(range(0,255))) )
    print( reqman.prettify( None ) )


def test_guess():
    b=bytes(range(0,255))
    assert reqman.guessValue("jijiij")=="jijiij"
    assert reqman.guessValue('jij"iij')=='jij"iij'
    assert reqman.guessValue('jij"i"ij')=='jij"i"ij'
    assert reqman.guessValue("jij'iij")=="jij'iij"
    assert reqman.guessValue("42")==42
    assert reqman.guessValue("2.5")==2.5
    assert reqman.guessValue("0.2")==.2
    assert reqman.guessValue("0.2.1")=="0.2.1"
    assert reqman.guessValue("null") == None
    assert reqman.guessValue("true") == True
    assert reqman.guessValue("false") == False
    assert reqman.guessValue("{}") == {}
    assert reqman.guessValue("[]") == []    

    assert reqman.guessValue(42)==42
    assert reqman.guessValue(b"h")==b"h"
    assert reqman.guessValue(str(b))==str(b)
    assert reqman.guessValue(None)==None    


import pytest,reqman

# same as 045
# same as 045
# same as 045
# same as 045, but with dynamic param (python param)

def test_complexe(Reqs): # ensure now is executed before upper
    y="""
- POST: /
  body: <<fichier|upper>>
  params:
    upper: return x.upper()
    now: |
      return "xxx"
    fichier: www<<now>>www
"""
    l=Reqs(y)
    assert len(l) == 1

    ll=l.execute( {"/" : (200,"ok")} ) # reqman.RMException: Can't find method 'NOW'
    assert ll[0].body == b"WWWXXXWWW" 

def test_complexe1(Reqs): # ensure now is executed before upper
    y="""
- PROC:
    - POST: /
      body: <<fichier|upper>>
      params:
        upper: return x.upper()
        now: return "xxx"
        fichier: www<<now>>www
- call: PROC
"""
    l=Reqs(y)
    assert len(l) == 1

    ll=l.execute( {"/" : (200,"ok")} ) # reqman.RMException: Can't find method 'NOW'
    assert ll[0].body == b"WWWXXXWWW" 

def test_complexe2(Reqs): # ensure now is executed before upper
    y="""
- PROC:
    - POST: /
      body: <<fichier|upper>>
      params:
        upper: return x.upper()
        fichier: www<<now>>www
- call: PROC
  params:
    now: return "xxx"
"""
    l=Reqs(y,trace=True)
    assert len(l) == 1

    ll=l.execute( {"/" : (200,"ok")} ) # reqman.RMException: Can't find method 'NOW'
    assert ll[0].body == b"WWWXXXWWW" 

def test_complexe3(Reqs): # ensure now is executed before upper
    y="""
- PROC:
    - POST: /
      body: <<fichier|upper>>
      params:
        fichier: www<<now>>www
- call: PROC
  params:
    now: return "xxx"
    upper: return x.upper()
"""
    l=Reqs(y)
    assert len(l) == 1

    ll=l.execute( {"/" : (200,"ok")} ) # reqman.RMException: Can't find method 'NOW'
    assert ll[0].body == b"WWWXXXWWW" 

def test_complexe4(Reqs): # ensure now is executed before upper
    y="""
- PROC:
    - POST: /
      body: <<fichier|upper>>
      params:
        upper: return x.upper()
- call: PROC
  params:
    fichier: www<<now>>www
    now: return "xxx"
"""
    l=Reqs(y)
    assert len(l) == 1

    ll=l.execute( {"/" : (200,"ok")} ) # reqman.RMException: Can't find method 'NOW'
    assert ll[0].body == b"WWWXXXWWW" 

def test_complexe5(Reqs): # ensure now is executed before upper
    y="""
- PROC:
    - POST: /
      body: <<fichier|upper>>
      params:
        now: return "xxx"
- call: PROC
  params:
    fichier: www<<now>>www
    upper: return x.upper()
"""
    l=Reqs(y)
    assert len(l) == 1

    ll=l.execute( {"/" : (200,"ok")} ) # reqman.RMException: Can't find method 'NOW'
    assert ll[0].body == b"WWWXXXWWW" 

def test_complexe6(Reqs): # ensure now is executed before upper
    y="""
- PROC:
    - POST: /
      body: <<fichier|upper>>
- call: PROC
  params:
    fichier: www<<now>>www
    now: return "xxx"
    upper: return x.upper()
"""
    l=Reqs(y,trace=True)
    assert len(l) == 1

    ll=l.execute( {"/" : (200,"ok")} ) # reqman.RMException: Can't find method 'NOW'
    assert ll[0].body == b"WWWXXXWWW" 




import pytest,reqman,json,yaml
import pytest,reqman,json

@pytest.mark.asyncio
async def test_hermetic_env():
    MOCK={
            "/pingpong": lambda m,p,b,h: (200,b)
        }
    
    r=reqman.Reqman()
    r.env["bytes"]=bytes(range(0,255))

    r.outputConsole = reqman.OutputConsole.FULL
    r.add("""
- POST: /pingpong
  body: Hello World
  tests:
    - status: 200
    - content: Hello

- POST: /pingpong
  body:
    a:
        - 1
        - 2
    b: "hello"
  tests:
    - status: 200
    - content: hello
    - json.a.0: 1
    - json.a.1: 2
    - json.b: "hello"


- POST: /pingpong
  body: 42
  tests:
    - status: 200
    - content: 42




- POST: /pingpong
  tests:
    - status: 200
    - content: ""

- POST: /pingpong
  body: null
  tests:
    - status: 200
    - content: ""




- POST: /pingpong
  body: <<bytes>>
  tests:
    - status: 200
    - content: ABCDEF       # test repr

- POST: /pingpong
  body: <<bytes>>
  tests:
    - status: 200
    - content: <<bytes>>    # test real bytes




- POST: /pingpong
  body: <<|giveBytes>>
  tests:
    - status: 200
    - content: ABCDEF       # test repr
  params:
    giveBytes: return b"ABCDEF"

- POST: /pingpong
  body: <<|giveBytes>>
  tests:
    - status: 200
    - content: <<|giveBytes>>   # test real bytes
  params:
    giveBytes: return b"ABCDEF"    

""")

    rr=await r.asyncExecute( paralleliz=False, http=MOCK )
    assert rr.code==0
    rr=await r.asyncExecute( paralleliz=True, http=MOCK )
    assert rr.code==0



import pytest,reqman

MOCK={"https://www.manatlan.com":(200,"ok")}

def test_static(Reqs):
    y="""
- yo:
    - yo2:
        - GET: https://www.manatlan.com
          foreach:
            - c: 10
            - c: 20
    - call: yo2
      foreach:
        - b: 10
        - b: 20
- call: yo
  foreach:
        - a: 10
        - a: 20
"""
    l=Reqs(y)
    assert len(l) == 1
    l.execute( MOCK )

def test_dynamic(Reqs):
    y="""
  - GET: https://www.manatlan.com
    foreach: <<liste>>
"""
    env=dict(liste=[dict(a=1),dict(a=2)])
    l=Reqs(y,env)
    assert len(l) == 1
    l.execute( MOCK )



def test_dynamic_bad(Reqs):
    y="""
  - GET: https://www.manatlan.com
    foreach: "nope"
"""
    env=dict(liste=[dict(a=1),dict(a=2)])
    
    l=Reqs(y,env)
    assert len(l) == 1
    
    with pytest.raises(reqman.RMException):
      l.execute( MOCK )

def test_dynamic_bad2(Reqs):
    y="""
  - GET: https://www.manatlan.com
    foreach: <<unknown>>
"""
    l=Reqs(y)
    assert len(l) == 1
    
    with pytest.raises(reqman.RMException):
      l.execute( MOCK )

def test_dynamic_bad3(Reqs):
    y="""
  - GET: https://www.manatlan.com
    foreach: <<nimp>>
"""
    env=dict(nimp="kkoo")

    l=Reqs(y,env)
    assert len(l) == 1
    
    with pytest.raises(reqman.RMException):
      l.execute( MOCK )

def test_dynamic_bad33(Reqs):
    y="""
  - GET: https://www.manatlan.com
    foreach: <<nimp>>
"""
    env=dict(nimp=None)

    l=Reqs(y,env)
    assert len(l) == 1
    
    with pytest.raises(reqman.RMException):
      l.execute( MOCK )

def test_dynamic_bad333(Reqs):
    y="""
  - GET: https://www.manatlan.com
    foreach: <<nimp>>
"""
    env=dict(nimp=False)

    l=Reqs(y,env)
    assert len(l) == 1
    
    with pytest.raises(reqman.RMException):
      l.execute( MOCK )

def test_dynamic_bad333(Reqs):
    y="""
  - GET: https://www.manatlan.com
    foreach: <<nimp>>
"""
    env=dict(nimp=42)

    l=Reqs(y,env)
    assert len(l) == 1
    
    with pytest.raises(reqman.RMException):
      l.execute( MOCK )

def test_dynamic_bad4(Reqs):
    y="""
  - GET: https://www.manatlan.com
    foreach: <<nimp>>
"""
    env=dict(nimp=dict(a=2))

    l=Reqs(y,env)
    assert len(l) == 1
    
    with pytest.raises(reqman.RMException):
      l.execute( MOCK )

def test_dynamic_bad5(Reqs):
    y="""
  - GET: https://www.manatlan.com
    foreach: <<nimp>>
"""
    env=dict(nimp=list("abc"))

    l=Reqs(y,env)
    assert len(l) == 1
    
    with pytest.raises(reqman.RMException):
      l.execute( MOCK )







import pytest,reqman,json

MOCK={"https://www.manatlan.com/IAmThePath":(200,"ok")}

def test_simplest_ko(Reqs):
    y="""
POST: https://www.manatlan.com/<<path_jo>>
headers:
    jo: <<header_jo>>
body: Hello <<val_jo>>
tests:
    - status: 404
"""
    l=Reqs(y)
    assert len(l) == 1
    ll= l.execute(MOCK)
    assert ll[0].url == "https://www.manatlan.com/<<path_jo>>"
    assert ll[0].inHeaders == {"jo": "<<header_jo>>"}
    # print(ll[0].body)
    # assert ll[0].body == b"Hello <<val_jo>>"
    assert ll[0].status==None
    assert not any(ll[0].tests)

def test_simplest_ok(Reqs):
    y="""
POST: https://www.manatlan.com/<<path_jo>>
headers:
    jo: <<header_jo>>
body: Hello <<val_jo>>
tests:
    - status: 200
    - content: ok
"""
    env=dict(
        path_jo="IAmThePath",
        header_jo="IAmTheHeader",
        val_jo="IAmTheContent",
    )
    l=Reqs(y,env)
    assert len(l) == 1
    ll= l.execute(MOCK)
    assert ll[0].url == "https://www.manatlan.com/IAmThePath"
    assert ll[0].inHeaders == {"jo": "IAmTheHeader"}
    assert ll[0].body == b"Hello IAmTheContent"
    assert any(ll[0].tests)

def test_save(Reqs):
    y="""
- GET: /a
  save: r
- GET: /b
"""
    l=Reqs(y)
    assert len(l) == 2
    ll=l.execute( {"/a":(200,"b"),"/b":(200,"ok")} )
    assert len(ll) == 2

    assert ll[0].url == "/a"
    assert ll[1].url == "/b"

    assert l.env["r"]=="b"

def test_tests(Reqs):
    y="""
- GET: /a
  tests:
    - status: 200
    - content: ok
"""
    l=Reqs(y)
    assert len(l) == 1
    ll=l.execute( {"/a":(200,"ok")} )
    assert len(ll) == 1
    assert all( ll[0].tests )

def test_tests_json(Reqs):
    y="""
- GET: /a
  tests:
    - status: 200
    - status: "200"
    - json.v: 42
    - json.v: "42"
"""
    l=Reqs(y)
    assert len(l) == 1
    ll=l.execute( {"/a":(200,json.dumps(dict(v=42)))} )
    assert len(ll) == 1
    assert all( ll[0].tests )

def test_tests_json(Reqs):
    y="""
- GET: /a
  tests:
    - json.v: <<v1>>
    - json.v: <<v2>>
  params:
    v1: 42
    v2: "42"
"""
    l=Reqs(y)
    assert len(l) == 1
    ll=l.execute( {"/a":(200,json.dumps(dict(v=42)))} )
    assert len(ll) == 1
    assert all( ll[0].tests )

def test_tests_json2(Reqs):
    y="""
- proc:
    - GET: /a
      tests:
        - json.v: <<v1>>
        - json.v: <<v2>>
- call: proc
  params:
    v1: 43
    v2: "43"
"""
    l=Reqs(y)
    assert len(l) == 1
    ll=l.execute( {"/a":(200,json.dumps(dict(v=43)))} )
    assert len(ll) == 1
    assert all( ll[0].tests )

def test_foreach_dynamic(Reqs):
    y="""
- proc:
    - GET: /a
      tests:
        - json.v: <<v1>>
        - json.v: <<v2>>
        - status: .!=111
        - status: .>100
        - status: .>=100
        - status: .<300
        - status: .<=300
        - status: 200
        - status: .=200
        - status: .==200
- call: proc
  foreach: <<dynlist>>
  params:
    v1: 43
    v2: "43"
"""
    l=Reqs(y, dict(dynlist=[{"i":1},{"i":2}])  )
    assert len(l) == 1
    ll=l.execute( {"/a":(200,json.dumps(dict(v=43)))} )
    assert len(ll) == 2
    assert all( ll[0].tests )
    assert all( ll[1].tests )

def test_foreach_static_from_main(Reqs):
    y="""
- proc:
    - GET: /a
      tests:
        - json.v: <<v1>>
        - json.v: <<v2>>
- call: proc
  foreach: <<staticlist>>
  params:
    v1: 43
    v2: "43"
    staticlist:
        - i: 1
        - i: 2
"""
    l=Reqs(y )
    assert len(l) == 1
    ll=l.execute( {"/a":(200,json.dumps(dict(v=43)))} )
    assert len(ll) == 2
    assert all( ll[0].tests )
    assert all( ll[1].tests )

def test_foreach_static_inside_proc(Reqs):
    y="""
- proc:
    - GET: /a
      foreach: <<staticlist>>
- call: proc
  params:
    staticlist:
        - i: 1
        - i: 2
"""
    l=Reqs(y )
    assert len(l) == 1
    ll=l.execute( {"/a":(200,json.dumps(dict(v=43)))} )
    assert len(ll) == 2

def test_check_hermetic_scope(Reqs):
    y="""
- proc:
    - GET: <<v>>
- call: proc
  params:
    v: /a

- proc2:
    - GET: <<vv>>
- call: proc2
  params:
    vv: /a
"""
    l=Reqs(y)
    assert len(l) == 2
    ll=l.execute( {"/a":(200,"ok")} )
    assert len(ll) == 2
    assert ll[0].scope=={'v': '/a'}
    assert ll[1].scope=={'vv': '/a'}


def test_check_hermetic_scope2(Reqs):
    y="""
- proc:
    - GET: <<v>>
      params:
        p: 1

- call: proc
  params:
    v: /a

- call: proc
  params:
    vv: /a
"""
    l=Reqs(y)
    assert len(l) == 2
    ll=l.execute( {"/a":(200,"ok")} )
    assert len(ll) == 2
    assert ll[0].scope=={'v': '/a', 'p': 1}
    assert ll[1].scope=={'vv': '/a',  'p': 1}

def test_check_hermetic_scope3(Reqs):
    y="""
- proc:
    - GET: <<v>>
      params:
        p: 1

- call: proc
  foreach:
    - v: /a

- call: proc
  foreach:
    - vv: /a
"""
    l=Reqs(y)
    assert len(l) == 2
    ll=l.execute( {"/a":(200,"ok")} )
    assert len(ll) == 2
    assert ll[0].scope=={'v': '/a', 'p': 1}
    assert ll[1].scope=={'vv': '/a',  'p': 1}

def test_check_hermetic_scope4(Reqs):
    y="""
- proc:
    - GET: <<v>>
      params:
        p: 1
    - GET: <<v>>
      params:
        p: 2
      foreach:
        - i: 1
        - i: 2
- call: proc
  foreach:
    - v: /a

- call: proc
  foreach:
    - v: /b
"""
    l=Reqs(y)
    assert len(l) == 2
    ll=l.execute( {"/a":(200,"ok"),"/b":(200,"ok")} )
    assert len(ll) == 6
    assert ll[0].scope=={'p': 1, 'v': '/a'}
    assert ll[1].scope=={'p': 2, 'v': '/a', 'i': 1}
    assert ll[2].scope=={'p': 2, 'v': '/a', 'i': 2}
    assert ll[3].scope=={'p': 1, 'v': '/b'}
    assert ll[4].scope=={'p': 2, 'v': '/b', 'i': 1}
    assert ll[5].scope=={'p': 2, 'v': '/b', 'i': 2}

    l=Reqs(y,trace=1)
    assert len(l) == 2
    ll=l.execute( {"/a":(200,"ok"),"/b":(200,"ok")} )



import reqman,pytest,sys,os

mock = {
  "http://a/start":(200,"ok"),
  "http://b/start":(200,"ok"),
  "http://a/hello":(200,"ok"),
  "http://b/hello":(200,"ok"),
}

def _continue(exe):
    with open("f.yml","w+") as fid:
        fid.write("""
        - GET: /hello
          doc: ici <<def>>
          tests:
            - status: 200
            - content: ok
        """)
        
    x=exe(".","--s",fakeServer=mock)  # generate a RMR
    assert "http://a/hello" in x.console
    assert x.rc==0
    assert x.console.count("content contains")==1
    assert os.path.isfile("reqman.html")
    os.unlink("reqman.html")
    lrmr=[i for i in os.listdir(".") if i.endswith(".rmr")]
    assert len(lrmr)==1
    os.unlink("reqman.conf")
    os.unlink("f.yml")
    # x.view()


    x=exe(lrmr[0],"--o:r1.html",fakeServer=mock) # rebuild HTML from a rmr -> single result
    assert "http://a/hello" not in x.console # because nothing is replayed
    assert x.rc==0
    assert os.path.isfile("r1.html")
    assert not x.console.count("content contains")==1 # nothing is replayed, only output

    x=exe(lrmr[0],"-other","--o:r2.html",fakeServer=mock) # replay RMR with another switch  -> single result
    assert "http://b/hello" in x.console
    assert "http://a/hello" not in x.console
    assert x.rc==0
    assert os.path.isfile("r2.html")
    assert x.console.count("content contains")==2

    x=exe(lrmr[0],"+other","--o:r3.html",fakeServer=mock) # compare RMR with another switch  -> dual result
    assert "http://b/hello" in x.console
    assert "http://a/hello" not in x.console #because A is not replayed
    assert x.rc==0
    assert os.path.isfile("r3.html")
    assert x.console.count("content contains")==2

    x=exe(lrmr[0],"--r","--o:r4.html",fakeServer=mock) # REPLAY the RMR, and compare old and new -> dual
    assert "http://b/hello" not in x.console
    assert "http://a/hello" in x.console
    assert x.rc==0
    assert os.path.isfile("r4.html")
    assert x.console.count("content contains")==1
    # x.view()

def test_COMMAND_rmr_tests_old_switches(exe):   #<- it's not a real test ... just COPY/PASTE this one for next tests

    with open("reqman.conf","w+") as fid:
        fid.write("""
        root: http://a
        def: A

        other:
          root: http://b
          def: B

          BEGIN:
          - GET: /start
            doc: ici <<def>>
            tests:
              - status: 200
              - content: ok

        """)
    _continue(exe)

def test_COMMAND_rmr_tests_new_switches(exe):   #<- it's not a real test ... just COPY/PASTE this one for next tests

    with open("reqman.conf","w+") as fid:
        fid.write("""
        root: http://a
        def: A

        switches:
          other:
            root: http://b
            def: B

            BEGIN:
            - GET: /start
              doc: ici <<def>>
              tests:
                - status: 200
                - content: ok
        """)
    _continue(exe)



import pytest,reqman,json

MOCK={
    "https://www.manatlan.com/utf8":(200,"oké"),
    "https://www.manatlan.com/cp1252":(200,"oké".encode().decode("cp1252")),
}

def test_same_encoding(Reqs):
    y="""
GET: https://www.manatlan.com/utf8
tests:
    - content: oké
"""
    l=Reqs(y)
    assert len(l) == 1
    ll= l.execute(MOCK)
    assert any(ll[0].tests)

def test_same_encoding2(Reqs):
    y="""
GET: https://www.manatlan.com/cp1252
tests:
    - content: oké
""".encode().decode("cp1252")
    l=Reqs(y)
    assert len(l) == 1
    ll= l.execute(MOCK)
    assert any(ll[0].tests)

def test_diff_encoding(Reqs):
    y="""
GET: https://www.manatlan.com/cp1252
tests:
    - content: oké
"""
    l=Reqs(y)
    assert len(l) == 1
    ll= l.execute(MOCK)
    assert any(ll[0].tests) #can't resolve ;-( (NEXT UPDATE !!! it's in the pipe)



import pytest
import reqman
import asyncio,sys
import contextlib,io,re,json,html
import tempfile,os,shutil


@pytest.fixture(scope="function")
def Reqs(request):
    def tester(*a,**k):
        return reqman.Reqs(*a,**k)

    yield tester


class FakeExeReturn():
    rc=0
    console=""
    rr=None
    def view(self):
        h=tempfile.mktemp()+".html"
        with open(h,"w+") as fid:
            ansi_escape =re.compile(r'(\x9B|\x1B\[)[0-?]*[ -\/]*[@-~]')
            console=ansi_escape.sub('', self.console)    
            fid.write("""<h3>Test file : "%s"</h3>""" % os.getenv('PYTEST_CURRENT_TEST') )
            fid.write("""<h3>RC : %s <-- "%s"</h3>""" % (self.rc," ".join(sys.argv)))
            fid.write("""<h3>Output Console:</h3><pre>%s</pre>""" % html.escape(console))
            if self.rr:
                fid.write("""<h3>Env:</h3><pre>%s</pre>""" % (json.dumps(self.rr.env, indent=4, sort_keys=True)))
                fid.write("""<h3>Output Html (%s):</h3>%s""" % (self.rr.__class__.__name__,self.rr.html))
            
        import webbrowser
        webbrowser.open_new_tab(h)   

@pytest.fixture(scope="function")
def exe(request):
    def tester(*a,fakeServer=None):
        sys.argv=["reqman.exe"]+list(a)


        f=FakeExeReturn()
        
        print(sys.argv)
        fo,fe = io.StringIO(),io.StringIO()
        with contextlib.redirect_stderr(fe):
            with contextlib.redirect_stdout(fo):
                rc=reqman.main(fakeServer=fakeServer,hookResults=f)

        output=fo.getvalue()+fe.getvalue()
        print(output)

        f.rc=rc
        f.console=output

        return f

    try:
        precdir = os.getcwd()
        dtemp = tempfile.mkdtemp()
        os.chdir( dtemp )

        yield tester

    finally:
        os.chdir( precdir )
        shutil.rmtree(dtemp)    



@pytest.yield_fixture(scope='session')
def event_loop(request):
    """Create an instance of the default event loop for each test case."""
    loop = asyncio.get_event_loop_policy().new_event_loop()
    yield loop
    loop.close()



import pytest, reqman, json
import datetime, pickle


def test_Env_pickable():
    e = reqman.Env({"a": 42})
    assert e == {"a": 42}
    assert pickle.loads(pickle.dumps(e)) == e

    e.save("b", 43)
    assert e == {"a": 42, "b": 43}
    assert pickle.loads(pickle.dumps(e)) == {"a": 42, "b": 43}


def test_Env_cant_save_existing():
    e = reqman.Env({"a": 42})
    assert e == {"a": 42}

    # with pytest.raises(reqman.RMException):
    assert e["a"] == 42
    e.save("a", 43)  # will erase old "a"
    assert (
        e["a"] == 43
    )  # important, to get token oauth first (in current major reqman's tests)!


def test_createEnv():
    assert reqman.Env() == {}
    assert reqman.Env({"a": 42}) == {"a": 42}

    assert reqman.Env(None) == {}
    assert reqman.Env("") == {}

    assert reqman.Env("a: 42") == {"a": 42}


def test_createEnvBad():
    assert reqman.Env(list("abc")) == {}
    assert reqman.Env("[1,2,3]") == {}
    # with pytest.raises(reqman.RMFormatException):
    #     reqman.Env( list("abc") )

    # with pytest.raises(reqman.RMFormatException):
    #     reqman.Env( "[1,2,3]" )

    with pytest.raises(reqman.RMFormatException):
        reqman.Env("- yolo\nbad: yaml")


def test_jpath():
    assert reqman.jpath({"v": 42}, "v") == 42
    assert reqman.jpath({"v": 42}, "v2") is reqman.NotFound

    env = reqman.Env(dict(toto=dict(v1=100, v2=None), tata=dict(l=["a", "b", "c"])))
    assert reqman.jpath(env, "titi") is reqman.NotFound
    assert reqman.jpath(env, "titi.size") is reqman.NotFound
    assert reqman.jpath(env, "titi.0") is reqman.NotFound

    assert reqman.jpath(env, "toto.v1") == 100
    assert reqman.jpath(env, "toto.v2") == None
    assert reqman.jpath(env, "toto.v3") is reqman.NotFound
    assert reqman.jpath(env, "toto.size") == 2

    assert reqman.jpath(env, "tata.l") == ["a", "b", "c"]
    assert reqman.jpath(env, "tata.l.1") == "b"
    assert reqman.jpath(env, "tata.l.1.size") == 1
    assert reqman.jpath(env, "tata.l.size") == 3


def test_jpath_python():
    env = dict(fct="return dict(a=dict(b=42))")
    assert reqman.jpath(env, "fct.size") == 1
    assert reqman.jpath(env, "fct.a.b") == 42


def test_simple():
    dt = datetime.datetime.now()

    env = reqman.Env(
        dict(
            s="world",
            i=42,
            f=3.14,
            t=(1, "kkk"),
            l=[1, "kkk"],
            d=dict(i=42),
            n=None,
            bt=True,
            bf=False,
            dt=dt,
            json={"b": "42", "content-type": "xxx"},
        )
    )
    assert env.replaceTxt("hello '<<>>'") == "hello '<<>>'"
    assert env.replaceTxt("hello '<<unknown>>'") == "hello '<<unknown>>'"
    assert env.replaceTxt("hello '<<s>>'") == "hello 'world'"
    assert env.replaceTxt("hello '<<i>>'") == "hello '42'"
    assert env.replaceTxt("hello '<<f>>'") == "hello '3.14'"
    assert env.replaceTxt("hello '<<t>>'") == """hello '[1, "kkk"]'"""
    assert env.replaceTxt("hello '<<l>>'") == """hello '[1, "kkk"]'"""
    assert env.replaceTxt("hello '<<d>>'") == """hello '{"i": 42}'"""
    assert env.replaceTxt("hello '<<n>>'") == """hello 'null'"""
    assert env.replaceTxt("hello '<<bt>>'") == """hello 'true'"""
    assert env.replaceTxt("hello '<<bf>>'") == """hello 'false'"""
    assert env.replaceTxt("hello '<<dt>>'") == """hello '%s'""" % str(dt)
    assert env.replaceTxt("hello '<<json.b>>'") == """hello '42'"""
    assert env.replaceTxt("hello '<<json.content-type>>'") == """hello 'xxx'"""


def test_complex():
    env = reqman.Env(dict(d=dict(v1=False, v2=None)))
    assert env.replaceTxt("hello '<<d.v1>>'") == "hello 'false'"
    assert env.replaceTxt("hello '<<d.v2>>'") == "hello 'null'"
    assert env.replaceTxt("hello '<<d.v3>>'") == "hello '<<d.v3>>'"


def test_rec():
    env = reqman.Env(dict(v="<<v1>>", v1="<<v2>>", v2="<<value>>", value="ok"))
    assert env.replaceTxt("hello '<<v>>'") == "hello 'ok'"


def test_rec2():
    env = reqman.Env(dict(d={"a": "<<c>>"}, c=42))
    assert env.replaceTxt("hello '<<d>>'") == """hello '{"a": 42}'"""


def test_transform_dynamix():
    env = reqman.Env(dict(v=42, m="return x*'*'"))
    assert (
        env.replaceTxt("hello '<<v|m>>'")
        == "hello '******************************************'"
    )


def test_transform_statix():
    env = reqman.Env(dict(m="return x*'*'"))
    with pytest.raises(reqman.RMPyException):
        env.replaceTxt("hello '<<42|m>>'")
    # assert env.replaceTxt("hello '<<42|m>>'") == "hello '******************************************'"


def test_transform_no_args_return_float():
    env = reqman.Env(dict(pi="return 3.14"))
    assert env.replaceTxt("hello '<<|pi>>'") == "hello '3.14'"


def test_transform_return_int():
    env = reqman.Env(dict(m="return int(x*7)"))
    with pytest.raises(reqman.RMPyException):
        env.replaceTxt("hello '<<42|m>>'")
        # assert env.replaceTxt("hello '<<42|m>>'") == "hello '294'"


def test_transform_return_bytes_and_replace_all():
    env = reqman.Env(dict(m="return b'bijour'"))
    assert env.replaceTxt("hello '<<|m>>'") == b"bijour"


def test_transform_unknown_method():
    env = reqman.Env()
    with pytest.raises(reqman.RMPyException):
        env.replaceTxt("hello '<<v|unknwon>>'")


def test_transform_bad_exception_in_method():
    env = reqman.Env(dict(m="return x/0"))
    with pytest.raises(reqman.RMPyException):
        env.replaceTxt("hello '<<42|m>>'")


def test_replaceObj():
    env = reqman.Env(dict(v=42, b=b"bytes", s="hello"))
    assert env.replaceObj(None) == None
    assert env.replaceObj("yo") == "yo"
    assert env.replaceObj(3.14) == 3.14
    assert env.replaceObj(42) == 42
    assert env.replaceObj(True) == True
    assert env.replaceObj(False) == False
    assert env.replaceObj({"a": "42"}) == {"a": "42"}
    assert env.replaceObj([{"a": "42"}]) == [{"a": "42"}]

    assert env.replaceObj("<<unknown>>") == "<<unknown>>"
    assert env.replaceObj("<<v>>") == 42
    assert env.replaceObj({"a": "<<v>>"}) == {"a": 42}

    assert env.replaceObj("<<b>>") == b"bytes"


def test_replaceObj2():
    env = reqman.Env(dict(obj1=dict(a=42, b="<<obj2>>"), obj2=dict(i=42, j="hello")))
    o = env.replaceObj("<<obj1>>")
    assert o == {"a": 42, "b": {"i": 42, "j": "hello"}}


def test_replaceObj3():
    env = reqman.Env(dict(obj1=dict(a=42, b="<<obj2>>"), obj2="hello"))
    o = env.replaceObj("<<obj1>>")
    assert o == {"a": 42, "b": "hello"}


def test_switches():
    env = reqman.Env(dict(obj1=dict(root="https://w1")))
    assert list(env.switches) == [("obj1", "https://w1")]


def test_renameKeys():
    dd = dict(KEY="kiki", a=42, b=dict(KEY=12, c=13))
    reqman.renameKeyInDict(dd, "KEY", "MyKey")
    assert dd == {"a": 42, "b": {"c": 13, "MyKey": 12}, "MyKey": "kiki"}


def test_HeadersMixed():
    dd = reqman.HeadersMixedCase(KeY="kiki")
    assert dd["KeY"] == "kiki"
    assert dd["key"] == "kiki"
    assert dd["key2"] == None
    assert dd.get("key") == "kiki"
    assert dd.get("key2") == None


## cf "test_300.new.py"
## cf "test_300.new.py"
## cf "test_300.new.py"
## cf "test_300.new.py"
def test_ultracomplex():
    env = reqman.Env(dict(user=dict(id=42, name="kiki"), user1="<<user>>"))
    assert env.replaceTxt("hello '<<user1.id>>'") == "hello '42'"


def test_ultracomplex2():
    env = reqman.Env(dict(user=dict(id=42, name="kiki"), user_id="<<user.id>>"))
    assert env.replaceTxt("hello '<<user_id>>'") == "hello '42'"

def test_ultracomplex3():
    env = reqman.Env(dict(user=dict(info=dict(id=42), name="kiki"), user1="<<user>>"))
    assert env.replaceTxt("hello '<<user1.info.id>>'") == "hello '42'"

def test_ultracomplex4():
    env = reqman.Env(dict(user=dict(info=dict(id=42), name="kiki"), info1="<<user.info>>"))
    assert env.replaceTxt("hello '<<info1.id>>'") == "hello '42'"



import pytest,reqman


def test_simplest(Reqs):
    y="""
- GET: /<<v|upper>>
  params:
    v: hello
    upper: return x.upper()
"""
    l=Reqs(y)
    assert len(l) == 1
    ll=l.execute( {"/HELLO" : (200,"ok")} )
    assert ll[0].url == "/HELLO"

def test_simplest_chaining(Reqs):
    y="""
- GET: /<<v|upper|lower>>
  params:
    v: Hello
    upper: return x.upper()
    lower: return x.lower()
"""
    l=Reqs(y)
    assert len(l) == 1
    ll=l.execute( {"/hello" : (200,"ok")} )
    assert ll[0].url == "/hello"

def test_simplest_transciant(Reqs):
    y="""
- GET: /<<param>>
  params:
    param: <<v|upper>>
    v: hello
    upper: return x.upper()
"""
    l=Reqs(y,trace=True)
    assert len(l) == 1
    ll=l.execute( {"/HELLO" : (200,"ok")} )
#     assert ll[0].url == "/HELLO"


def test_pass_object_to_method(Reqs):
    y="""
- GET: /<<param>>
  params:
    param: <<data|getVal>>
    data:
        value: "hello"
        nimp: 42
    getVal: return x.get("value")
"""
    l=Reqs(y,trace=True)
    ll=l.execute( {"/hello" : (200,"ok")} )
    assert ll[0].url == "/hello"


def test_pass_object_to_method_jwt(Reqs):
    y="""
- GET: /hello
  headers:
    Authorization: Bearer <<data|createJwt>>
  params:
    data:
        value: "hello"
        nimp: 42
    createJwt: |
        import jwt
        return jwt.encode(data,"").decode()
"""
    l=Reqs(y,trace=True)
    ll=l.execute( {"/hello" : (200,"ok")} )
    assert ll[0].url == "/hello"




import reqman,pytest,sys

def test_COMMAND_bad(exe):
    x=exe("new","gfdfds")
    assert x.rc==-1
    assert "you shoul provide a full url" in x.console


def test_COMMAND_new(exe):
    x=exe("new","http://jim/jo1")
    assert x.rc==0
    assert "Create reqman.conf" in x.console
    assert "Create 0010_test.rml" in x.console

    assert "GET: /jo1" in open("0010_test.rml").read()
    assert "root: http://jim" in open("reqman.conf").read()

    x=exe("new","http://jim/jo2?kif=1")
    assert "Use 'reqman.conf" in x.console
    assert "Create 0020_test.rml" in x.console
    assert "GET: /jo2?kif=1" in open("0020_test.rml").read()

    x=exe("new","/jo3")
    assert "Use 'reqman.conf" in x.console
    assert "Create 0030_test.rml" in x.console
    assert "GET: /jo3" in open("0030_test.rml").read()
    



import reqman,pytest,sys,os


def test_COMMAND_usage_no_files(exe):
    x=exe(".")
    assert "ERROR COMMAND: no yml files found" in x.console
    assert "USAGE" in x.console
    assert "pre-made 'switch' defined" in x.console
        
def test_COMMAND_usage_list_switches(exe):
    with open("reqman.conf","w+") as fid:
        fid.write("root: 'https://s1.com'\n")
        fid.write("sw1:\n")
        fid.write("    root: 'https://s2.com'\n")
        
    x=exe(".")
    assert "Use 'reqman.conf" in x.console
    assert "ERROR COMMAND: no yml files found" in x.console
    assert "USAGE" in x.console
    assert "-sw1 :" in x.console




import pytest,reqman


def test_simplest(Reqs):
    y="""
- proc:
    - GET: https://www.manatlan.com
      params:
        p1: 1
- call: proc
  params:
        p2: 2

"""
    l=Reqs(y)
    assert len(l) == 1
    assert l[0].reqs[0].params ==  {'p1': 1}

def test_simplest2(Reqs):
    y="""
- proc:
    - GET: https://www.manatlan.com/<<p1>>/<<p>>/<<p2>>
      params:
        p1: p1
- call: proc
  foreach:
    - p: 1
    - p: 2
  params:
    p2: p2

"""
    l=Reqs(y)
    assert len(l) == 1
    assert l[0].foreach == [{'p': 1}, {'p': 2}] # foreach params
    assert l[0].reqs[0].params == {'p1': 'p1'}

    MOCK={"https://www.manatlan.com/p1/1/p2":(200,"ok"),"https://www.manatlan.com/p1/2/p2":(200,"ok")}
    ll=l.execute( MOCK )
    assert ll[0].status==200
    assert ll[1].status==200
    assert ll[0].url == "https://www.manatlan.com/p1/1/p2"
    assert ll[1].url == "https://www.manatlan.com/p1/2/p2"



import pytest,reqman


def test_simplest(Reqs):
    y="""
- yo:
    - GET: https://www.manatlan.com
- call: yo
"""
    l=Reqs(y)
    assert len(l) == 1

def test_simplest1(Reqs):
    y="""
- yo:
    - GET: https://www.manatlan.com
- call:
    - yo
"""
    l=Reqs(y)
    assert len(l) == 1

def test_simplest1(Reqs):
    y="""
- yo:
    - GET: https://www.manatlan.com
- call:
    - yo
    - yo
"""
    l=Reqs(y)
    assert len(l) == 2

def test_simplest2(Reqs):
    y="""
- yo:
    - GET: https://www.manatlan.com
- call: yo
- GET: https://www.manatlan.com
"""
    l=Reqs(y)
    assert len(l) == 2

def test_simplest3(Reqs):
    y="""
- yo:
    - GET: https://www.manatlan.com
    - GET: https://www.manatlan.com
- call: yo
"""
    l=Reqs(y)
    assert len(l) == 1
    assert type(l[0]) is reqman.ReqGroup

def test_simplest4(Reqs):
    y="""
- yo2:
    - GET: https://www.manatlan.com
- yo:
    - GET: https://www.manatlan.com
    - call: yo2
- call: yo
"""
    l=Reqs(y)
    assert len(l) == 1
    assert type(l[0]) is reqman.ReqGroup

def test_simplest5(Reqs):
    y="""
- yo:
    - yo2:
        - GET: https://www.manatlan.com
    - GET: https://www.manatlan.com
    - call: yo2
- call: yo
"""
    l=Reqs(y)
    assert len(l) == 1
    assert type(l[0]) is reqman.ReqGroup


def test_empty(Reqs):
    y="""
- yo2:
    - GET: https://www.manatlan.com
    - GET: https://www.manatlan.com
"""
    l=Reqs(y)
    assert len(l) == 0

def test_empty2(Reqs):
    y="""
- yo2:
    - GET: https://www.manatlan.com
    - "shit happens here"
"""
    l=Reqs(y)
    assert len(l) == 0    

def test_bad_2action(Reqs):
    y="""
    - jo:
        - GET: https://www.manatlan.com
    - GET: https://www.manatlan.com
      call: jo
    """
    with pytest.raises(reqman.RMFormatException):
        Reqs(y)
        
def test_bad(Reqs):
    y="""
- call: yo
"""
    with pytest.raises(reqman.RMFormatException):
        Reqs(y)

def test_bad2(Reqs):
    y="""
- yo:
    - GET: https://www.manatlan.com
    - "shit happens here"
- call: yo
"""
    with pytest.raises(reqman.RMFormatException):
        Reqs(y)

def test_bad3(Reqs):
    y="""
- yo:
    - GET: https://www.manatlan.com
- yo:
    - GET: https://www.manatlan.com
- call: yo
"""
    with pytest.raises(reqman.RMFormatException):
        Reqs(y)

def test_bad4(Reqs):
    y="""
- yo:
    - GET: https://www.manatlan.com
- call: 
    yo: yo
"""
    with pytest.raises(reqman.RMFormatException):
        Reqs(y)

def test_bad5(Reqs):
    y="""
- yo:
    - GET: https://www.manatlan.com
- call: 
"""
    with pytest.raises(reqman.RMFormatException):
        Reqs(y)



import pytest,json

mock={
    "https://www.manatlan.com/hello":(200,"ok"),
}

def test_normal(Reqs):
    y="""
- GET: https://www.manatlan.com/<<v>>
  params:
    v: return "hello"
  tests:
    - status: 200
"""
    l=Reqs(y)
    assert len(l) == 1
    ll=l.execute( mock )
    assert all(ll[0].tests)


def test_bytes(Reqs):
    y="""
- GET: https://www.manatlan.com/<<bytes|str>>
  params:
    bytes: return b"hello"
    str: return x.decode()
  tests:
    - status: 200
"""
    l=Reqs(y)
    assert len(l) == 1
    ll=l.execute( mock )
    assert all(ll[0].tests)

def test_bytes_dyn_param(Reqs):
    y="""
- POST: https://www.manatlan.com/hello
  body: <<bytes>>
  params:
    bytes: return b"hello"
  tests:
    - status: 200
"""
    l=Reqs(y)
    assert len(l) == 1
    ll=l.execute( mock )
    assert all(ll[0].tests)

def test_bytes_method(Reqs):
    y="""
- POST: https://www.manatlan.com/hello
  body: <<|bytes>>
  params:
    bytes: return b"hello"
  tests:
    - status: 200
"""
    l=Reqs(y)
    assert len(l) == 1
    ll=l.execute( mock )
    assert all(ll[0].tests)



import pytest,reqman


def test_empty(Reqs):
    y="""
- POST: https://www.manatlan.com
  body:
"""
    l=Reqs(y)
    assert len(l) == 1
    assert l[0].body is None

def test_null(Reqs):
    y="""
- POST: https://www.manatlan.com
  body: null
"""
    l=Reqs(y)
    assert len(l) == 1
    assert l[0].body is None

def test_string(Reqs):
    y="""
- POST: https://www.manatlan.com
  body: "hello"
"""
    l=Reqs(y)
    assert len(l) == 1
    assert l[0].body == "hello"

def test_dict(Reqs):
    y="""
- POST: https://www.manatlan.com
  body: 
    a: 12
    b: 13
"""
    l=Reqs(y)
    assert len(l) == 1
    assert l[0].body == {'a': 12, 'b': 13}

def test_list(Reqs):
    y="""
- POST: https://www.manatlan.com
  body: 
    - line1
    - line2
"""
    l=Reqs(y)
    assert len(l) == 1
    assert l[0].body == ["line1","line2"]

def test_bool(Reqs):
    y="""
- POST: https://www.manatlan.com
  body: true
"""
    l=Reqs(y)
    assert len(l) == 1
    assert l[0].body is True

def test_number(Reqs):
    y="""
- POST: https://www.manatlan.com
  body: 42
"""
    l=Reqs(y)
    assert len(l) == 1
    assert l[0].body == 42

def test_float(Reqs):
    y="""
- POST: https://www.manatlan.com
  body: 3.14
"""
    l=Reqs(y)
    assert len(l) == 1
    assert l[0].body == 3.14

def test_tuple_is_string(Reqs):
    y="""
- POST: https://www.manatlan.com
  body: (12,34)
"""
    l=Reqs(y)
    assert len(l) == 1
    assert l[0].body == '(12,34)'


# def test_bad(Reqs):
#     y="""
# - GET: https://www.manatlan.com
#   headers: "nimp"
# """
#     with pytest.raises(reqman.RMFormatException):
#         Reqs(y)


import pytest, reqman, json

class Ex:
    def __init__(self,id):
        self.id=id
    def __repr__(self):
        return str(self.id)
    def __eq__(self,o):
        return (o and self.id==o.id)

    
def test_1():
    assert Ex(id=1) == Ex(id=1)
    assert Ex(id=1) != Ex(id=2)
    assert Ex(id=1) != None


def test_zip1():
    ex1 = [
        Ex(id=1),
    ]

    ex2 = [
    ]    

    assert "[(1, None)]" == str(reqman.izip(ex1,ex2))    
    assert not reqman.comparable(reqman.izip(ex1,ex2))

def test_zip2():
    ex1 = [
    ]

    ex2 = [
        Ex(id=1),
    ]    

    assert "[(None, 1)]" == str(reqman.izip(ex1,ex2))    
    assert not reqman.comparable(reqman.izip(ex1,ex2))

def test_zip():
    ex1 = [
        Ex(id=1),
        Ex(id=2),
        Ex(id=3),
    ]

    ex2 = [
        Ex(id=1),
        Ex(id=3),
        Ex(id=4),
        Ex(id=5),
    ]    

    assert "[(1, 1), (2, None), (3, 3), (None, 4), (None, 5)]" == str(reqman.izip(ex1,ex2))
    assert reqman.comparable(reqman.izip(ex1,ex2))

    ex1 = [
        Ex(id=1),
        Ex(id=3),
        Ex(id=4),
        Ex(id=5),
    ]    

    ex2 = [
        Ex(id=1),
        Ex(id=2),
        Ex(id=3),
    ]

    assert "[(1, 1), (None, 2), (3, 3), (4, None), (5, None)]" == str(reqman.izip(ex1,ex2))
    assert reqman.comparable(reqman.izip(ex1,ex2))


def test_zip_same():
    ex1 = [
        Ex(id=1),
        Ex(id=1),
        Ex(id=1),
    ]

    ex2 = [
        Ex(id=1),
        Ex(id=1),
        Ex(id=1),
    ]    

    assert "[(1, 1), (1, 1), (1, 1)]" == str(reqman.izip(ex1,ex2))
    assert reqman.comparable(reqman.izip(ex1,ex2))

def test_zip_nimp():
    ex1 = [
        Ex(id=1),
    ]

    ex2 = [
        Ex(id=2),
    ]    

    assert "[(1, None), (None, 2)]" == str(reqman.izip(ex1,ex2))
    assert not reqman.comparable(reqman.izip(ex1,ex2))



import reqman,pytest,sys

def test_COMMAND_no_args(exe):
    r=exe()
    assert r.rc==-1
    assert "ERROR COMMAND: no yml files found" in r.console
    assert "USAGE" in r.console

def test_COMMAND_bad_option(exe):
    r=exe(".","--dsqdqs")
    assert r.rc==-1
    assert "ERROR COMMAND: bad option" in r.console
    assert "USAGE" in r.console

def test_COMMAND_bad_option2(exe):
    r=exe(".","--ko","--jkjkj")
    assert r.rc==-1
    assert "ERROR COMMAND: bad option" in r.console
    assert "USAGE" in r.console

def test_COMMAND_bad_option_try_help(exe):
    r=exe(".","--help")
    # r.view()
    assert r.rc==-1
    assert "ERROR COMMAND: bad option" in r.console
    assert "USAGE" in r.console

def test_EXECUTION_unknown_file(exe):
    r=exe("fdsfdsqgfds")
    assert r.rc==-1
    assert "ERROR EXECUTION: bad param" in r.console
    assert "USAGE" not in r.console

def test_EXECUTION_bad_switch(exe):
    with open("test.yml","w+") as fid:
        fid.write("")
    r=exe(".","-unknown")
    assert r.rc==-1
    assert "ERROR COMMAND: bad switch" in r.console
    assert "USAGE" in r.console






import reqman, asyncio,pytest
from pprint import pprint
import json

MOCK={
    "/ok": (200,"ok"),
}


def test_ko(exe):

    with open("f.yml","w+") as fid:
        fid.write("""
- GET: /<<var>>
  tests:
    - status: 404
""")

    x=exe(".","--o",fakeServer=MOCK)
    # x.view()
    assert x.rc == 1 #1 error coz the 404 test is non playable


def test_ok(exe):

    with open("f.yml","w+") as fid:
        fid.write("""
- GET: /<<var>>
  tests:
    - status: 200
    - content: ok
""")

    x=exe(".","var:ok","--o",fakeServer=MOCK)
    # x.view()
    assert x.rc == 0







import pytest,reqman


def test_simplest(Reqs):
    y="""
- GET: https://www.manatlan.com
  headers:
    jo: hello
"""
    l=Reqs(y)
    assert len(l) == 1
    assert type(l[0]) is reqman.Req
    assert l[0].headers == {'jo': 'hello'}

def test_simplest2(Reqs):
    y="""
- yo:
    - GET: https://www.manatlan.com
      headers:
        jo: hello
- call: yo
  headers:
    jo: hello2
"""
    l=Reqs(y)
    assert len(l) == 1
    assert type(l[0]) is reqman.ReqGroup    
    assert l[0].reqs[0].headers == {'jo': 'hello2'}

def test_headers_as_list_and_tests_as_dict(Reqs): #bad practice
    y="""
- yo:
    - GET: https://www.manatlan.com
      headers:
        - jo: hello
      tests:
        status: 200
- call: yo
  headers:
    - jo: hello2
"""
    l=Reqs(y)
    assert len(l) == 1
    assert type(l[0]) is reqman.ReqGroup    
    assert l[0].reqs[0].headers == {'jo': 'hello2'}

def test_empty(Reqs):
    y="""
- GET: https://www.manatlan.com
  headers:
"""
    l=Reqs(y)
    assert len(l) == 1
    assert l[0].headers == {}

def test_bad(Reqs):
    y="""
- GET: https://www.manatlan.com
  headers: "nimp"
"""
    with pytest.raises(reqman.RMFormatException):
        Reqs(y)



def test_2simplest(Reqs):
    y="""
- GET: https://www.manatlan.com
  headers:
    jo: fsdgfdsgfds
"""
    l=Reqs(y)
    assert len(l) == 1

def test_2simplest2(Reqs):
    y="""
- proc:
    - GET: https://www.manatlan.com
- call: proc
  headers:
    jo: fsdgfdsgfds
"""
    l=Reqs(y)
    assert len(l) == 1

def test_2simplest3(Reqs): # bad header's ext, but not called -> forget
    y="""
- proc:
    - GET: https://www.manatlan.com
      Headers:
         jo: fsdgfdsgfds
"""
    l=Reqs(y)
    assert len(l) == 0



def test_2bad(Reqs):
    y="""
- GET: https://www.manatlan.com
  Headers:
    jo: fsdgfdsgfds
"""
    with pytest.raises(reqman.RMFormatException):
        Reqs(y)

def test_2bad2(Reqs):
    y="""
- proc:
    - GET: https://www.manatlan.com
- call: proc
  Headers:
    jo: fsdgfdsgfds
"""
    with pytest.raises(reqman.RMFormatException):
        Reqs(y)

def test_2bad3(Reqs):
    y="""
- proc:
    - GET: https://www.manatlan.com
      Headers:
          jo: fsdgfdsgfds
- call: proc
"""
    with pytest.raises(reqman.RMFormatException):
        Reqs(y)

def test_2bad4(Reqs):
    y="""
- proc:
    - YO: https://www.manatlan.com
- call: proc
"""
    with pytest.raises(reqman.RMFormatException):
        Reqs(y)

def test_2bad_actionExt1(Reqs):
    y="""
- GET: https://www.manatlan.com
  headers: "yolo"
"""
    with pytest.raises(reqman.RMFormatException):
        Reqs(y)

def test_2bad_actionExt2(Reqs):
    y="""
- proc:
    - GET: https://www.manatlan.com
      headers: "yolo"
- call: proc
"""
    with pytest.raises(reqman.RMFormatException):
        Reqs(y)

def test_header_override_remove(Reqs):
    y="""
- yo:
    - GET: https://www.manatlan.com
      headers:
        jo: hello
- call: yo
  headers:
    jo: null
"""
    l=Reqs(y)
    assert len(l) == 1
    assert type(l[0]) is reqman.ReqGroup    
    assert l[0].reqs[0].headers ==  {'jo': None}  # before execution

    ll=l.execute( {"https://www.manatlan.com":(200,"ok")})
    assert len(ll) == 1
    assert ll[0].inHeaders=={}



import reqman, asyncio, pytest
from pprint import pprint
import json

# 07/12/2020
# la recopi de dict ne marche pas comme je voudra
# je l'ai decouvert la, et voici un TU qui devrait
# normalement passe ...
# mais c pas le cas a l'heure actuelle


MOCK = {"/a/42": (200, "ok")}


def test_recopi_dico(exe):

    with open("reqman.conf", "w+") as fid:
        fid.write(
            """
#--------------------------
# test a simple value
#--------------------------
id: 42

id1: <<id>>

#--------------------------
# test a dict value
#--------------------------
user:
    id: 42
    name : kiki

user1: <<user>>

"""
        )

    with open("f.yml", "w+") as fid:
        fid.write(
            """
- GET: /a/<<id>>
  doc:  value=<<id>>
  tests:
    - status: 200

- GET: /a/<<id1>>
  doc:  value=<<id1>>
  tests:
    - status: 200

- GET: /a/<<user.id>>
  doc:  value=<<user.id>>
  tests:
    - status: 200

- GET: /a/<<user1.id>>
  doc:  value=<<user1.id>>
  tests:
    - status: 200

"""
        )

    x = exe(".", "--o", fakeServer=MOCK)
    # print(x.console)
    # x.view()
    assert x.rc == 0



import sys,os,io,contextlib
from glob import glob
import pytest
import fakereqman, reqman
import os,tempfile,shutil

WS=None
def setup_module():
    global WS
    WS=fakereqman.FakeWebServer(11111)
    WS.start()
    import time
    time.sleep(1)   


def teardown_module():
    global WS
    WS.stop()

F=os.path.join(os.path.dirname(os.path.dirname(__file__)),"REALTESTS")
@pytest.mark.parametrize('file', glob( os.path.join( F,"auto_*.yml")) + glob( os.path.join( F,"auto_*/*.yml")) )
def test_file(file):
    err=run(file)
    assert err=="", "File '%s' : %s" % (file,err)


def run(params):
    err=""
    fo,fe = io.StringIO(),io.StringIO()
    with contextlib.redirect_stderr(fe):
        with contextlib.redirect_stdout(fo):
            for err in fakereqman.main(params,avoidBrowser=True):
                if err:
                    break

    output=fo.getvalue()+fe.getvalue()
    print(output)
    return err




import reqman, pytest, sys, os


def test_COMMAND_complex(exe):
    mock = {"/hello": (200, "ok")}

    with open("f.yml", "w+") as fid:
        fid.write(
            """
        - GET: /<<var.a.b>>
          tests:
            - status: 200
            - content: ok
          params:
            var: |
                d={}
                d["a"]={"b":"hello"}
                return d
        """
        )

    x = exe(".", fakeServer=mock)
    # x.view()
    assert x.rc == 0


def test_COMMAND_complex2(exe):
    mock = {"/HELLO": (200, "ok")}

    with open("f.yml", "w+") as fid:
        fid.write(
            """
        - GET: /<<var.a.b|upper>>
          tests:
            - status: 200
            - content: ok
          params:
            var: |
                d={}
                d["a"]={"b":"hello"}
                return d
            upper:
              return x.upper()
        """
        )

    x = exe(".", fakeServer=mock)
    assert x.rc == 0


def test_priority_token(
    exe
):  # current major reqman's tests !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
    # PROBABLY THE MOST IMPORTANT TEST ;-)
    # (BASE FEATURE FROM DAY ONE)

    with open("reqman.conf", "w+") as fid:
        fid.write(
            """
          token: fake

          BEGIN:
            - GET: /token 
              save: token
        """
        )

    f = """
      - GET: /<<token>>
        tests:
          status: 200
      - GET: /<<token>>
        params:
          token: real2  #override temporaly
        tests:
          status: 201
      - GET: /<<token>>
        tests:
          status: 200
    """
    with open("f1.yml", "w+") as fid:
        fid.write(f)
    with open("f2.yml", "w+") as fid:
        fid.write(f)

    MOCK = {"/token": (200, "real"), "/real": (200, "ok"), "/real2": (201, "ok")}
    x = exe(".", fakeServer=MOCK)
    # x.view()
    assert x.rc == 0


def test_env_hermetic_between_files(exe):  # pato trouble
    MOCK = {"/first": (200, "second"), "/second": (200, "ok")}

    with open("f1.yml", "w+") as fid:
        fid.write(
            """
        - GET: /first
          save: data
        - GET: /<<data>>
          tests:
            - status: 200
        """
        )

    with open("f2.yml", "w+") as fid:
        fid.write(
            """
        - GET: /<<data>>
          tests:
            - status: 404
        """
        )

    x = exe(".", fakeServer=MOCK)
    # x.view()
    assert x.rc == 1  # 1 error coz the 404 test is non playable


def test_ensure_global_proc_doesnt_save_globally(exe):
    MOCK = {"/first": (200, "second"), "/second": (200, "ok")}

    with open("reqman.conf", "w+") as fid:
        fid.write(
            """
          GLOBAL:
            - GET: /first
              save: data
        """
        )

    with open("f1.yml", "w+") as fid:
        fid.write(
            """
        - call: GLOBAL
        - GET: /<<data>>
          tests:
            - status: 200
        """
        )

    with open("f2.yml", "w+") as fid:
        fid.write(
            """
        - GET: /<<data>>
          tests:
            - status: 404
        """
        )

    x = exe(".", fakeServer=MOCK)
    # x.view()
    assert x.rc == 1  # 1 error coz the 404 test is non playable



import pytest,reqman


def test_break(Reqs):
    y="""
- GET: https://www.manatlan.com
- break
- GET: https://www.manatlan.com
"""
    l=Reqs(y)
    assert len(l) == 1

def test_break_in_proc(Reqs):
    y="""
- Tests:
    - GET: https://www.manatlan.com
    - break
    - GET: https://www.manatlan.com
- call: Tests
- GET: https://www.manatlan.com
"""
    l=Reqs(y)
    assert len(l) == 2



import pytest,reqman,pickle

def test_reqs_pickable(Reqs):
    y="""
GET: https://www.manatlan.com
"""
    l=Reqs(y)
    assert len(l) == 1

    r=l[0]
    rr=pickle.loads(pickle.dumps(r))
    assert rr.method == r.method
    assert rr.path == r.path
    assert rr.body == r.body
    assert rr.doc == r.doc
    assert rr.saves == r.saves
    assert rr.headers == r.headers
    # assert rr.parent == r.parent

    ll=pickle.loads(pickle.dumps(l))
    assert len(ll)==len(l)
    assert ll[0].path==l[0].path

def test_simplest(Reqs):
    y="""
GET: https://www.manatlan.com
"""
    l=Reqs(y)
    assert len(l) == 1

def test_simplest2(Reqs):
    y="""
- GET: https://www.manatlan.com
"""
    l=Reqs(y)
    assert len(l) == 1

def test_bad_2verbs(Reqs):
    y="""
    - GET: https://www.manatlan.com
      POST: https://www.manatlan.com
    """
    with pytest.raises(reqman.RMFormatException):
        Reqs(y)


def test_bad(Reqs):
    y="""hello"""
    with pytest.raises(reqman.RMFormatException):
        Reqs(y)

def test_bad2(Reqs):
    y="""
- GET: https://www.manatlan.com
- hhjfdhhfdjs
"""
    with pytest.raises(reqman.RMFormatException):
        Reqs(y)

def test_bad3(Reqs):
    y="""
- FUCK: https://www.manatlan.com
"""
    with pytest.raises(reqman.RMFormatException):
        Reqs(y)

def test_bad4(Reqs):
    y="""
- GET: 
    nimp: nimp
"""
    with pytest.raises(reqman.RMFormatException):
        Reqs(y)
        
def test_bad5(Reqs):
    y="""
- GET:
"""
    with pytest.raises(reqman.RMFormatException):
        Reqs(y)

def test_bad6(Reqs):
    y="""
- GET: hgfdhgfd
jo: malformed
"""
    with pytest.raises(reqman.RMFormatException):
        Reqs(y)

def test_bad7(Reqs):
    y="""
ffdsqfds
"""
    with pytest.raises(reqman.RMFormatException):
        Reqs(y)


def test_empty(Reqs):
    l=Reqs("")
    assert len(l) == 0


import reqman, asyncio,pytest
from pprint import pprint
import json
s= dict(int=42,str="hello")

MOCK={
    "/xml": (200,"""<?xml version="1.0" encoding="UTF-8"?>
<x xmlns:ns2="www">
    <entete>
        <ns2:typeDocument>hello</ns2:typeDocument>
    </entete>
    <a v="1">aaa1</a>
    <a>aaa2</a>
    <b v="9">b9</b>
    <b v="11">b11</b>
    <c>yolo <i>xxx</i></c>
</x>"""),
    "/noxml": (200,"""jkjhhjjhhhjhjhjjhjggfgffg"""),
    "/aaa2": (200,"ok")
}


def test_xpath_ok(exe):

    with open("f.yml","w+") as fid:
        fid.write("""
- GET: /xml
  tests:
    - status: 200
    - json.result: null
    - xml.//c.0 : "yolo xxx"
    - xml.//c.0.size   : 8
    - xml.//a: ["aaa1","aaa2"]
    - xml.//a[1].0: "aaa1"
    - xml.//a[last()].0: "aaa2"
    - xml.//a/text(): ["aaa1","aaa2"]
    - xml.//a.size : .=2
    - xml.//a[@v].0: "aaa1"
    - xml.//a[@v]/@v.0: 1
    - xml.//a|//b: ["aaa1","aaa2","b9","b11"]
    - xml.//a|//b.size: 4
    - xml.//*:nooooo: null
    - xml.//ns2:typeDocument.0: hello
    - xml.//c: .>1                       # bad
    
""")

    x=exe(".","--o",fakeServer=MOCK)
    # print(x.console)
    # x.view()
    assert x.rc == 1  # the bad one ^^



def test_xpath_ko(exe):

    with open("f.yml","w+") as fid:
        fid.write("""
- GET: /noxml
  tests:
    - status: 200
    - json.result: null
    - xml.//nooooo: null
""")

    x=exe(".","--o",fakeServer=MOCK)
    # print(x.console)
    # x.view()
    assert x.rc == 0


def test_xpath_compute(exe):

    with open("f.yml","w+") as fid:
        fid.write("""
- GET: /xml
  tests:
    - status: 200
  save:
    redirect:  <<xml.//a[last()]>>

- GET: /<<redirect.0>>
  tests:
    - status: 200

""")

    x=exe(".","--o",fakeServer=MOCK)
    # print(x.console) 
    # x.view()
    assert x.rc == 0

def test_xpath_compute2(exe):

    with open("f.yml","w+") as fid:
        fid.write("""
- GET: /xml
  tests:
    - status: 200
  save:
    redirect:  <<xml.//a[last()].0>>

- GET: /<<redirect>>
  tests:
    - status: 200

""")

    x=exe(".","--o",fakeServer=MOCK)
    # print(x.console) 
    # x.view()
    assert x.rc == 0




import pytest,reqman


def test_simplest(Reqs):
    y="""
- GET: https://www.manatlan.com
  foreach:
    - a: 12
    - a: 20
"""
    l=Reqs(y)
    assert len(l) == 1
    assert type(l[0]) is reqman.ReqGroup

def test_simplest2(Reqs):
    y="""
- yo:
    - GET: https://www.manatlan.com
- call: yo
  foreach:
        - a: 12
        - a: 20
"""
    l=Reqs(y)
    assert len(l) == 1
    assert type(l[0]) is reqman.ReqGroup

# def test_print_reqs(Reqs):
#     y="""
# - GET: /<<i>>
#   tests:
#     - status: 200
#   foreach:
#     - i: 1
#     - i: 2
# """
#     l=Reqs(y,name="mon_fichier.yml")
#     assert len(l) == 1
#     out="""Reqs's Name: mon_fichier.yml
# <ReqGroup foreach:[{'i': 1}, {'i': 2}] scope:{}>
#   <Req GET: /<<i>>>
#   	tests: [{'status': 200}]"""
#     assert repr(l) == out
#     l.execute( {"/1":(200,"ok"),"/2":(200,"ok")} )
#     assert repr(l) == out

def test_simplest3(Reqs):
    y="""
- yo:
    - GET: https://www.manatlan.com
      foreach:
        - b: 10
        - b: 20
- call: yo
  foreach:
        - a: 10
        - a: 20
"""
    l=Reqs(y)
    assert len(l) == 1
    # assert l[-1].params == {'b': 20, 'a': 20}


def test_empty(Reqs):
    y="""
- GET: https://www.manatlan.com
  foreach: #does nothing
"""
    l=Reqs(y)
    assert len(l) == 1

def test_bad(Reqs):
    y="""
- GET: https://www.manatlan.com
  foreach:
    a: 12
"""
    with pytest.raises(reqman.RMFormatException):
        Reqs(y)


import reqman, asyncio,pytest
from pprint import pprint

# def test_solo1():
#     r=reqman.ReqmanCommand( r"examples/dual" )
#     rr=r.execute( )
#     with open("/home/manatlan/aeff1.html","w+") as fid:
#         fid.write( reqman.render(rr))

# def test_solo2():
#     r=reqman.ReqmanCommand( r"examples/dual" )
#     rr=r.execute( ["site1"] )
#     with open("/home/manatlan/aeff2.html","w+") as fid:
#         fid.write( reqman.render(rr))


# @pytest.mark.asyncio
# async def test_dual():
#     r=reqman.ReqmanCommand( r"examples/dual" )
#     rr=await r.asyncExecuteDual( [],["site1"] )
#     with open("/home/manatlan/aeff.html","w+") as fid:
#         fid.write( reqman.render(rr))

MOCK={
    "http://a/1":(200,"ok"),
    "http://a/2":(200,"ok"),
    "http://a/3":(200,"ok"),
    "http://b/1":(201,"ok"),
    "http://b/2":(201,"ok"),
    "http://b/3":(201,"ok"),
}

def test_dual_izip(exe):  

    with open("reqman.conf","w+") as fid:
        fid.write("""
        root: http://a
        def: A

        switches:
          other:
            root: http://b
            def: B
        """)

    with open("f.yml","w+") as fid:
        fid.write("""
- GET: /<<i>>
  doc: "call <<i>>"
  foreach:
    - i: 1
    - i: 2
    - i: 3
  tests:
    - content: ok

""")
        
    x=exe(".",fakeServer=MOCK)
    assert x.rc == 0

    x=exe(".","+other",fakeServer=MOCK)
    assert x.rc == 0
    # x.view()



import pytest, reqman, json
import datetime,pickle


def test_Env_dyn():
    env=reqman.Env( {"a":42} )
    assert env == {"a":42}

    env2=env.clone()
    assert env2 == {"a":42}

    env2.save("yolo",13)

    assert env2["yolo"] == 13
    assert "yolo" not in env


def test_Env_dyn_jpath():
    env=reqman.Env( {"orig":dict(x=42)} )
    assert reqman.jpath(env,"orig.x") == 42

    env.save("data", dict(x=42))
    assert reqman.jpath(env,"data.x") == 42
    assert reqman.jpath(env,"data.size") == 1



import reqman, asyncio,pytest
from pprint import pprint
import json
s= dict(int=42,str="hello")

MOCK={
    "/a": (200,json.dumps(s)),
    "/ok/ok/ok/ok": (200,"ok"),
}


def test_savvve(exe):

    with open("reqman.conf","w+") as fid:
        fid.write("""
testByte: |
    return "ok" if type(x)==bytes else "ko"
testInt: |
    return "ok" if type(x)==int else "ko"
testStr: |
    return "ok" if type(x)==str else "ko"
""")

    with open("f.yml","w+") as fid:
        fid.write("""
- GET: /a
  tests:
    - status: 200
    - json.int: 42
    - json.str: hello
  save:
    isOkS: <<status|testInt>>
    isOk: <<content|testByte>>
    isOk2: <<json.int|testInt>>
    isOk3: <<json.str|testStr>>

- GET: /<<isOkS>>/<<isOk>>/<<isOk2>>/<<isOk3>>
  tests:
    status: 200

""")

    x=exe(".","--o",fakeServer=MOCK)
    # print(x.console)
    # x.view()
    assert x.rc == 0





import pytest,reqman,json

MOCK={
    "https://www.manatlan.com/utf8":(200,"oké"),
    "https://www.manatlan.com/cp1252":(200,"oké".encode().decode("cp1252")),
}

def test_same_encoding(exe):  
    y="""
GET: https://www.manatlan.com/utf8
tests:
    - content: oké
"""

    with open("f.yml","w+") as fid:
        fid.write(y)
        
    x=exe(".",fakeServer=MOCK)
    assert x.rc == 0

def test_same_encoding2(exe): 
    y="""
GET: https://www.manatlan.com/cp1252
tests:
    - content: oké
""".encode().decode("cp1252")

    with open("f.yml","w+") as fid:
        fid.write(y)
        
    x=exe(".",fakeServer=MOCK)
    assert x.rc == 0

def test_diff_encoding(exe): 
    y="""
GET: https://www.manatlan.com/cp1252
tests:
    - content: oké
"""

    with open("f.yml","w+") as fid:
        fid.write(y)
        
    x=exe(".",fakeServer=MOCK)
    assert x.rc == 0
    # x.view()

def test_diff_encoding_with_rconf(exe): 
    y="""
GET: https://www.manatlan.com/cp1252
tests:
    - content: <<val>>
"""

    with open("f.yml","w+") as fid:
        fid.write(y)

    with open("reqman.conf","w+") as fid:
        fid.write("""val: oké""")
        
    x=exe(".",fakeServer=MOCK)
    assert x.rc == 0
    # x.view()

def test_diff_encoding_with_rconf2(exe): 
    y="""
- GET: https://www.manatlan.com/utf8
  tests:
    - content: <<val>>

- GET: https://www.manatlan.com/cp1252
  tests:
    - content: <<val>>
""".encode().decode("cp1252")

    with open("f.yml","w+") as fid:
        fid.write(y)

    with open("reqman.conf","w+") as fid:
        fid.write("""val: oké""".encode().decode("cp1252"))
        
    x=exe(".",fakeServer=MOCK)
    assert x.rc == 0
    # x.view()



import pytest, reqman, json
import datetime,pickle


xml="""<?xml version="1.0" encoding="UTF-8"?>
<x xmlns:ns2="www">
    <entete>
        <ns2:typeDocument>hello</ns2:typeDocument>
    </entete>
    <a v="1">aaa1</a>
    <a>aaa2</a>
    <b v="9">b9</b>
    <b v="11">b11</b>
    <c>yolo <i>xxx</i></c>
</x>"""

def test_soapresponse():
    xx="""<?xml version="1.0" encoding="UTF-8"?>
<SOAP-ENV:Envelope xmlns:SOAP-ENV="http://schemas.xmlsoap.org/soap/envelope/">
    <SOAP-ENV:Header>header</SOAP-ENV:Header>
    <SOAP-ENV:Body>
        <result>hello</result>
        <result2 xmlns="myns">hello2</result2>
    </SOAP-ENV:Body>
</SOAP-ENV:Envelope>"""
    x=reqman.Xml(xx)
    assert x.xpath("//result")==["hello"]
    assert x.xpath("//result/text()")==["hello"]
    assert x.xpath("//SOAP-ENV:Header")==["header"]
    assert x.xpath("//*:Header")==["header"]
    assert x.xpath("//*[local-name()='result2']")==["hello2"]
    assert x.xpath("//*:result2")==["hello2"]

def test_xpath():
    x=reqman.Xml(xml)
    assert x.xpath("//unknown")==reqman.NotFound
    assert x.xpath("//c")==["yolo xxx"]
    assert x.xpath("//b[@v>10]")==["b11"]
    assert x.xpath("//b[@v<10]")==["b9"]
    assert x.xpath("//b[@v]")==["b9","b11"]
    assert x.xpath("//ns2:typeDocument")==["hello"]
    assert x.xpath("//a")==["aaa1","aaa2"]
    assert x.xpath("//a/text()")==["aaa1","aaa2"]
    assert x.xpath("//a[1]")==["aaa1"]
    assert x.xpath("//a[2]")==["aaa2"]
    assert x.xpath("//a[last()-1]/text()")==["aaa1"]
    assert x.xpath("//a/@v")==["1"]
    assert x.xpath("//b/@v")==['9', '11']
    assert x.xpath("//a|//b")==['aaa1', 'aaa2', 'b9', 'b11']

def test_simple():
    env = reqman.Env(
        dict(
            xml=reqman.Xml(xml),
            s="world",
            upper= "return x.upper()",
        )
    )
    assert env.replaceTxt("hello '<<xml.//a[1].0>>'") == "hello 'aaa1'"
    assert env.replaceTxt("hello '<<s>>'") == "hello 'world'"
    assert env.replaceTxt("hello '<<s|upper>>'") == "hello 'WORLD'"
    assert env.replaceTxt("hello '<<xml.//a[1].0|upper>>'") == "hello 'AAA1'" # can't work with xpath '|' !!!!!!!!!!!!!!
    assert "<entete>" in env.replaceTxt("hello '<<xml>>'")

    print(env["xml"])




import pytest,reqman,json
import reqman, asyncio, pytest

@pytest.mark.asyncio
async def test_nimp():

    
    conf="""
BEGIN:
    - GET: /check
      tests:
        - status: 200

proc1: return x*"a"

val_bb: bb
v: 2

dua:
    root: http://machinedua.com

"""
    r=reqman.Reqman(conf)

    assert r.switches==[("dua","http://machinedua.com")]

    r.add("""
- call: BEGIN
- GET: /<<i>>
  tests:
    - status: 200
  foreach:
    - i: 1
    - i: 2
""")

    r.add("""
- GET: /<<v|proc1>>
  tests:
    - status: 200
- POST: /<<v|proc2>>
  tests:
    - status: 200
  params:
    proc2: return x*"a"
- PUT: /<<val_bb>>
  tests:
    - status: 200
""")
    mock={"/":(200,"ok"),"/bb":(200,"ok"),"/aa":(200,"ok"),"/check":(200,"ok"),"/1":(200,"ok"),"/2":(200,"ok")}
    rr=await r.asyncExecute( paralleliz=True, http=mock )
    assert rr.code==0

        
    ll=await r.asyncExecute( paralleliz=False, http=mock )
    assert rr.code==0



import pytest,reqman,json

mock={
    "/start" : (200,json.dumps(dict(route=dict(second="next"),nb=42))),
    "/go/next" : (200,"ok"),
}


def test_save(Reqs):
    y="""
- GET: /a
  save: r
- GET: /<<r>>
"""
    l=Reqs(y)
    assert len(l) == 2
    ll=l.execute( {"/a":(200,"b"),"/b":(200,"ok")} )
    assert len(ll) == 2

    assert ll[0].url == "/a"
    assert ll[1].url == "/b"

def test_save_new(Reqs):
    y="""
- GET: /a
  save:
    r: <<content>>
- GET: /<<r>>

- GET: /a
  save:
    r: <<status>>
- GET: /<<r>>


- GET: /j
  save:
    r: <<json.v>>
- GET: /<<r>>

"""
    l=Reqs(y)
    assert len(l) == 6
    ll=l.execute( {"/a":(200,"b"),"/b":(200,"ok"),"/200":(200,"ok"),"/j":(200,json.dumps({"v":"b"}))} )
    assert len(ll) == 6

    assert ll[0].url == "/a"
    assert ll[1].url == "/b"
    
    assert ll[2].url == "/a"
    assert ll[3].url == "/200"

    assert ll[4].url == "/j"
    assert ll[5].url == "/b"

def test_save_new_VIA_CALL(Reqs):
    y="""
- PROC:
    - GET: /a
      save:
        r: <<content>>
    - GET: /<<r>>

    - GET: /a
      save:
        r: <<status>>
    - GET: /<<r>>


    - GET: /j
      save:
        r: <<json.v>>
    - GET: /<<r>>

- call: PROC
"""
    l=Reqs(y)
    assert len(l) == 1
    ll=l.execute( {"/a":(200,"b"),"/b":(200,"ok"),"/200":(200,"ok"),"/j":(200,json.dumps({"v":"b"}))} )
    assert len(ll) == 6

    assert ll[0].url == "/a"
    assert ll[1].url == "/b"
    
    assert ll[2].url == "/a"
    assert ll[3].url == "/200"

    assert ll[4].url == "/j"
    assert ll[5].url == "/b"

def test_save_1(Reqs):
    env=reqman.Env( """
PROC:
    - GET: /<<first>>
      doc: fonction commune
      tests:
        - status: 200
""")

    y="""
- call: PROC
  params:
    first: start
  save: data
- POST: /go/<<data.route.second>>
  tests:
    - status: 200
    - content: ok
"""

    
    l=Reqs(y,env)    
    ll=l.execute( mock )
    assert len(ll)==2
    assert all(ll[0].tests)
    assert all(ll[1].tests)


def test_save_2(Reqs):
    env=reqman.Env( """
PROC:
    - GET: /<<first>>
      doc: fonction commune
      tests:
        - status: 200
""")
    y2="""
- context:
    - call: PROC
      params:
        first: start
      save: 
        redirige: <<json.route.second>>

    - POST: /go/<<redirige>>
      tests:
        - status: 200
        - content: ok

- call: context
"""

    l=Reqs(y2,env,trace=True)    
    ll=l.execute( mock )
    assert len(ll)==2
    assert all(ll[0].tests)
    assert all(ll[1].tests)



import reqman,pytest,sys,os

def test_COMMAND_output_reqman_html(exe):
    with open("f.yml","w+") as fid:
        fid.write("""
        - GET: http://localhost:8080/
        """)
    assert not os.path.isfile("reqman.html")
        
    x=exe(".")

    assert os.path.isfile("reqman.html")

def test_COMMAND_no_output_html(exe):
    with open("f.yml","w+") as fid:
        fid.write("""
        - GET: http://localhost:8080/
        """)
    assert not os.path.isfile("reqman.html")
    x=exe(".","--o")
    assert "---------------------------------------------------------------------------" in x.console # ensure display FULL HTTP TRACE in console
    assert not os.path.isfile("reqman.html")

def test_COMMAND_output_anotherFile(exe):
    with open("f.yml","w+") as fid:
        fid.write("""
        - GET: http://localhost:8080/
        """)
    assert not os.path.isfile("reqman.html")
    x=exe(".","--o:kiki.html")
    assert not os.path.isfile("reqman.html")
    assert os.path.isfile("kiki.html")

def test_COMMAND_only_ko(exe):
    mock = {"/hello":(200,"ok")}

    with open("f.yml","w+") as fid:
        fid.write("""
        - GET: /hello
          tests:
            - status: 200
            - content: ok
        """)
        
    x=exe(".","--k",fakeServer=mock)
    assert x.rc == 0
    assert x.console.strip().startswith("RESULT:")  # there is no KO, no output (except result: %d/%d)

    x=exe(".",fakeServer=mock)
    assert x.rc == 0
    assert not x.console.strip().startswith("RESULT:")  # there is no KO, but output (minimal one) !!
    assert "TEST:" in x.console



def test_COMMAND_shebang(exe):   #<- it's not a real test ... just COPY/PASTE this one for later tests
    mock = {"/hello":(200,"ok")}

    with open("f.yml","w+") as fid:
        fid.write("""#! --k
        - GET: /hello
          tests:
            - status: 200
            - content: ok
        """)
        
    x=exe("--i","f.yml",fakeServer=mock) # should call A file only !
    assert os.path.isfile("reqman.html")
    assert x.rc == 0
    assert "Use SHEBANG : " in x.console
    assert "TEST:" not in x.console # ensure no test are displayed (--ko)


def test_COMMAND_fakeServer(exe):   #<- it's not a real test ... just COPY/PASTE this one for next tests
    mock = {"/hello":(200,"ok")}

    with open("f.yml","w+") as fid:
        fid.write("""
        - GET: /hello
          tests:
            - status: 200
            - content: ok
        """)
        
    x=exe(".",fakeServer=mock)
    assert os.path.isfile("reqman.html")
    assert x.rc == 0




import reqman, asyncio, pytest
from pprint import pprint

MOCK = {
    "/a": lambda method, url, body, headers: (200, "ok")
    if headers.get("auth") == "ok"
    else (404, "no"),
    "/yes": (200, "ok"),
}


def test_replace_headers_in_rc(exe):

    with open("reqman.conf", "w+") as fid:
        fid.write(
            """
user1:
  auth: "ok"
  x-autre2: hello2

headers:
  <<user1>>
"""
        )

    with open("f.yml", "w+") as fid:
        fid.write(
            """
- GET: /a
  headers: 
    auth: null #override
    x-autre: hello
  tests:
    - status: 404
- GET: /a
  headers:
    auth: "ko"
    x-autre: hello
  tests:
    - status: 404
- GET: /a
  headers:
    x-autre: hello
  tests:
    - status: 200
"""
        )

    x = exe(".", "--o", fakeServer=MOCK)
    assert x.rc == 0


def test_bad_headers_in_rc(exe):

    with open("reqman.conf", "w+") as fid:
        fid.write(
            """
user1: 412
  
headers:
  <<user1>>
"""
        )

    with open("f.yml", "w+") as fid:
        fid.write(
            """
- GET: /yes
  tests:
    - status: 200
"""
        )

    x = exe(".", "--o", fakeServer=MOCK)
    # x.view()
    assert x.rc == 1  # 1 error


def test_bad_headers2_in_rc(exe):

    with open("reqman.conf", "w+") as fid:
        fid.write(
            """
 
headers:
  <<user1>>
"""
        )

    with open("f.yml", "w+") as fid:
        fid.write(
            """
- GET: /yes
  tests:
    - status: 200
"""
        )

    x = exe(".", "--o", fakeServer=MOCK)
    assert x.rc == 1  # 1 error


def test_replace_headers2_in_rc(exe):

    with open("reqman.conf", "w+") as fid:
        fid.write(
            """

token: "no"

headers:
  auth: <<token>>

switchs:
    mod:
        token: "ok"

    headers:
        auth: <<token>>


"""
        )

    with open("f.yml", "w+") as fid:
        fid.write(
            """
- GET: /a
  headers: 
    auth: null #override
    x-autre: hello
  tests:
    - status: 404
- GET: /a
  headers:
    auth: "ko"
    x-autre: hello
  tests:
    - status: 404
- GET: /a
  headers:
    x-autre: hello
  tests:
    - status: 200
"""
        )

    x = exe(".", "-mod", "--o", fakeServer=MOCK)
    # x.view()
    assert x.rc == 0



import reqman, asyncio,pytest
from pprint import pprint

MOCK={
    "/a": lambda method,url,body,headers: 12/0,
    "/b": (200,"ok"),
}


def test_500(exe):

    with open("f.yml","w+") as fid:
        fid.write("""
- GET: /a
  tests:
    - status: 500
""")

    x=exe(".","--o",fakeServer=MOCK)
    #x.view()
    assert x.rc == 0

def test_error_in_py_call(exe):

    with open("f.yml","w+") as fid:
        fid.write("""
- POST: /b
  body:
    id:   1
    content: <<txt|b64>>
  tests:
    - status: 201
    - json.ok: true
  params:
    txt: hello world
    b64: |
        import base64
        return base64.b64encode( x )

""")

    x=exe(".","--o",fakeServer=MOCK)
    assert x.rc == 2




from reqman import json,Env


def test_priority_dont_override_local_value(Reqs):
    MOCK={
        "/a":(200,"ok"),
        "/b":(201,"ok"),
    }
    y="""
    - proc:
        - GET: /<<p>>
          params:
            p: a     

    - call: proc # call a
      tests:
        status: 200

    - call: proc # call a  (pato will not be happy ;-) )
      params:
        p: b
      tests:
        status: 200
    """

    ll=Reqs(y,trace=True).execute(MOCK)
    # "b" is NOT priority over "a"
    for i in ll:
      assert all(i.tests)    

def test_create_default_value_for_proc(Reqs): # hey pato, look here
    MOCK={
        "/a":(200,"ok"),
        "/b":(201,"ok"),
    }
    y="""
    - proc:
        - GET: /<<p|defa>>
          params:
            defa: return x or "a"

    - call: proc # call a
      tests:
        status: 200

    - call: proc # call b  (pato will be happy ;-) )
      params:
        p: b
      tests:
        status: 201
    """

    ll=Reqs(y).execute(MOCK)
    # "b" is priority over "a"
    for i in ll:
      assert all(i.tests)    


def test_priority_over_env(Reqs):
    y="""
    - GET: /<<var>>
      tests:
        status: 200

    - GET: /<<var>>
      tests:
        status: 201
      params:
        var: second   # override current 'var'

    - GET: /<<var>>
      tests:
        status: 200

    """
    MOCK={
        "/first":(200,'ok'),
        "/second":(201,'ok'),
    }
    ll=Reqs(y,{"var":"first"},trace=True).execute(MOCK)
    for i in ll:
      assert all(i.tests)
    


def test_priority_over_saved(Reqs):
    y="""

    - GET: /first
      tests:
        status: 200
        json.val: bad
      save:
        var: <<json.val>>

    - GET: /<<var>>
      tests:
        status: 404

    - GET: /<<var>>
      tests:
        status: 200
      params:
        var: second   # override current 'var'

    - GET: /<<var>>
      tests:
        status: 404

    """
    MOCK={
        "/first":(200,json.dumps( {"val":"bad"} )),
        "/second":(200,"ok"),
    }
    ll=Reqs(y).execute(MOCK)
    for i in ll:
      assert all(i.tests)
    
    


import pytest,reqman,json
import reqman, asyncio, pytest


MOCK={
        "/first":(200,"path"),
        "/path/a":(200,"ok"),

        "/value":(200,"ok"),
        "/ok":(200,"ok"),
    }

CONF="""
BEGIN:
    - GET: /first
      tests:
        - status: 200
      save: token

headers:
  test: This is a global header
"""

@pytest.mark.asyncio
async def test_hermetic_env_paralleliz():
    
    r=reqman.Reqman(CONF)
    r.outputConsole = reqman.OutputConsole.FULL

    r.add("""
- GET: /<<token>>/a
  tests:
    - status: 200

- GET: /<<suite>>
  tests:
    - status: 404

- proc:
    - proc2:
        - GET: /value
          tests:
            - status: 200
          save: suite
        
        - GET: /<<suite>>
          tests:
            - status: 200
    - call: proc2

- call: proc

- GET: /<<suite>>
  tests:
    - status: 200

""")

    rr=await r.asyncExecute( paralleliz=True, http=MOCK )
    assert rr.code==1 #1 error coz the 404 test is non playable

@pytest.mark.asyncio
async def test_hermetic_env():
    
    r=reqman.Reqman(CONF)
    r.outputConsole = reqman.OutputConsole.FULL

    r.add("""
- GET: /<<token>>/a
  tests:
    - status: 200

- GET: /<<suite>>
  tests:
    - status: 404

- proc:
    - proc2:
        - GET: /value
          tests:
            - status: 200
          save: suite
        
        - GET: /<<suite>>
          tests:
            - status: 200
    - call: proc2

- call: proc

- GET: /<<suite>>
  tests:
    - status: 200

""")

    rr=await r.asyncExecute( paralleliz=False, http=MOCK )
    assert rr.code==1 #1 error coz the 404 test is non playable



import reqman, asyncio,pytest
from pprint import pprint
import json


MOCK={
    "/list": (200,json.dumps( dict(fakeresult=3) )),
    "/3": (200,"ok"),
}


def test_scenario_with_non_resolved(exe):

    with open("f.yml","w+") as fid:
        fid.write("""
- GET: /list
  tests:
    - status: 200
  save:
    next: <<json.result>>   # "result" doesn't exists

- GET: /<<next>>
  tests:
    status: 200
    content: ok

- GET: /3
  tests:
    status: 200
    content: ok

- GET: /4
  tests:
    status: 404

""")

    x=exe(".","--o",fakeServer=MOCK)
    # print(x.console)
    # x.view()
    assert x.rc == 2 # 2 errors !





import reqman,pytest,sys,os
import contextlib,io

def execfile(filepath, globals=None, locals=None):
    if globals is None:
        globals = {}
    globals.update({
        "__file__": filepath,
        "__name__": "__main__",
    })
    with open(filepath, 'rb') as file:
        exec(compile(file.read(), filepath, 'exec'), globals, locals)

def test_exe(): #ensure that the module is executable (thru its "if __name__=="__main__":)
    sys.argv=[""]
    fo,fe = io.StringIO(),io.StringIO()
    with pytest.raises(SystemExit):
        with contextlib.redirect_stderr(fe):
            with contextlib.redirect_stdout(fo):
                execfile("reqman.py")
    output=fo.getvalue()+fe.getvalue()      
    assert "USAGE" in output
    assert reqman.__version__ in output

        



import reqman, asyncio, pytest

@pytest.mark.asyncio
async def test():
    yml="""
- GET: /a
  tests:
    status: 200
    content: ok
"""
    env={}
    mock={"/a":(200,"ok")}

    r= await reqman.testContent(yml,env,mock) 

    assert r.html
    assert r.code==0
    assert r.ok==2
    assert r.total==2



import pytest,reqman

#TODO: a ventiler dans les autres

def test_simplest(Reqs):
    y="""
- GET: https://www.manatlan.com
  headers:
    jo: fsdgfdsgfds
"""
    l=Reqs(y)
    assert len(l) == 1

def test_simplest2(Reqs):
    y="""
- proc:
    - GET: https://www.manatlan.com
- call: proc
  headers:
    jo: fsdgfdsgfds
"""
    l=Reqs(y)
    assert len(l) == 1

def test_simplest3(Reqs): # bad header's ext, but not called -> forget
    y="""
- proc:
    - GET: https://www.manatlan.com
      Headers:
         jo: fsdgfdsgfds
"""
    l=Reqs(y)
    assert len(l) == 0



def test_bad(Reqs):
    y="""
- GET: https://www.manatlan.com
  Headers:
    jo: fsdgfdsgfds
"""
    with pytest.raises(reqman.RMFormatException):
        Reqs(y)

def test_bad2(Reqs):
    y="""
- proc:
    - GET: https://www.manatlan.com
- call: proc
  Headers:
    jo: fsdgfdsgfds
"""
    with pytest.raises(reqman.RMFormatException):
        Reqs(y)

def test_bad3(Reqs):
    y="""
- proc:
    - GET: https://www.manatlan.com
      Headers:
          jo: fsdgfdsgfds
- call: proc
"""
    with pytest.raises(reqman.RMFormatException):
        Reqs(y)

def test_bad4(Reqs):
    y="""
- proc:
    - YO: https://www.manatlan.com
- call: proc
"""
    with pytest.raises(reqman.RMFormatException):
        Reqs(y)

def test_bad_actionExt1(Reqs):
    y="""
- GET: https://www.manatlan.com
  headers: "yolo"
"""
    with pytest.raises(reqman.RMFormatException):
        Reqs(y)

def test_bad_actionExt2(Reqs):
    y="""
- proc:
    - GET: https://www.manatlan.com
      headers: "yolo"
- call: proc
"""
    with pytest.raises(reqman.RMFormatException):
        Reqs(y)
